#include <iostream>
#include "mraa.hpp"
#include "led_interface.hpp"
#include "th02.hpp"
//-------------------------------------------------------------------------------------------------
float calculate_optimal_current(float ambient_temperature) {
	float ret_value = (0.003881554 * ambient_temperature * ambient_temperature) + (0.626921775 * ambient_temperature) + 477.6894677;
	return ret_value;
}
//-------------------------------------------------------------------------------------------------
int temp_feedback() {

	unsigned char recent_luminance = 0;
	float recent_temperature = 0;
	float recent_current = 0;
	float recent_voltage = 0;
	float optimal_current = 0;

	led_interface::led_interface ledif;
	th02::th02 trh;

	while (1) {
		std::cout << "Connecting to Szasza's controller..." << std::endl;
		try {
			ledif.check_connection();
			std::cout << "Connected to Szasza's controller" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(1);
	}

	while (1) {
		std::cout << "Connecting to TH02 sensor..." << std::endl;
		try {
			trh.check_sensor();
			std::cout << "Connected to TH02 sensor" << std::endl;
			break;
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
		sleep(1);
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	while (1) {

		// Acquire recent luminance
		while (1) {
			try {
				recent_luminance = ledif.get_luminance();
				break;
			} catch (...) {
				sleep(1);
			}
		}


		// Acquire recent temperature
		while (1) {
			try {
				recent_temperature = trh.get_temp(1000);
				break;
			} catch (...) {
				sleep(1);
			}
		}

		// Acquire recent current
		while (1) {
			try {
				recent_current = ledif.get_i(4);
				break;
			} catch (...) {
				sleep(1);
			}
		}

		// Acquire recent voltage
		while (1) {
			try {
				recent_voltage = ledif.get_u(4);
				break;
			} catch (...) {
				sleep(1);
			}
		}

		// Calculate optimal forward current
		optimal_current = calculate_optimal_current(recent_temperature);

		// Send new luminance value
		if ( recent_current < optimal_current ) {
			if ( recent_luminance <= 253 ) {
				while(1) {
					try {
						ledif.set_luminance(recent_luminance + 1);
						break;
					} catch (...) {
						sleep(1);
					}
				}
			}
		}
		else {
			if ( recent_luminance >= 1 ) {
				while (1) {
					try {
						ledif.set_luminance(recent_luminance - 1);
						break;
					} catch (...) {
						sleep(1);
					}
				}
			}
		}

		std::cout << "Recent temperature: " << recent_temperature << std::endl;
		std::cout << "Recent current: " << recent_current << " mA, optimal: " << optimal_current << " mA" << std::endl;
		std::cout << "Recent voltage: " << recent_voltage << " V" << std::endl;
		std::cout << "Luminance adjusted to: " << (unsigned)recent_luminance << std::endl;
		std::cout << "----------------------------------------------------" << std::endl;

		sleep(2);

	}

	return 0;

}
//-------------------------------------------------------------------------------------------------
